@@ -1,7 +1,6 @@
 <?php
-declare(strict_types = 1);
 
-namespace TYPO3\CMS\Adminpanel\ModuleApi;
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -16,6 +15,8 @@ namespace TYPO3\CMS\Adminpanel\ModuleApi;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Adminpanel\ModuleApi;
+
 /**
  * ModuleDataStorageCollection is an object storage for adminpanel modules and their data
  */
@@ -26,7 +27,7 @@ class ModuleDataStorageCollection extends \SplObjectStorage
         $this->attach($module, $moduleData);
     }
 
-    public function getHash($object)
+    public function getHash(object $object): string
     {
         if ($object instanceof ModuleInterface) {
             return $object->getIdentifier();
