@@ -1,7 +1,6 @@
 <?php
-declare(strict_types = 1);
 
-namespace TYPO3\CMS\Core\Resource\Event;
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -16,6 +15,8 @@ namespace TYPO3\CMS\Core\Resource\Event;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Core\Resource\Event;
+
 use TYPO3\CMS\Core\Resource\FileInterface;
 
 /**
@@ -23,17 +24,9 @@ use TYPO3\CMS\Core\Resource\FileInterface;
  *
  * Event listeners can clean up third-party references with this event.
  */
-final class BeforeFileDeletedEvent
+final readonly class BeforeFileDeletedEvent
 {
-    /**
-     * @var FileInterface
-     */
-    private $file;
-
-    public function __construct(FileInterface $file)
-    {
-        $this->file = $file;
-    }
+    public function __construct(private FileInterface $file) {}
 
     public function getFile(): FileInterface
     {
