@@ -18,6 +18,4 @@ namespace TYPO3\CMS\Core\Resource\Exception;
 /**
  * An exception to throw if a file does not exist
  */
-class FileDoesNotExistException extends ResourceDoesNotExistException
-{
-}
+class FileDoesNotExistException extends ResourceDoesNotExistException {}
