@@ -27,11 +27,8 @@ class DashboardPresetRegistry implements SingletonInterface
     /**
      * @var DashboardPreset[]
      */
-    private $dashboardPresets = [];
+    private array $dashboardPresets = [];
 
-    /**
-     * @return array
-     */
     public function getDashboardPresets(): array
     {
         if (empty($this->dashboardPresets)) {
@@ -45,16 +42,13 @@ class DashboardPresetRegistry implements SingletonInterface
             );
 
             return [
-                'dashboardPreset-fallback' => $fallbackDashboardPreset
+                'dashboardPreset-fallback' => $fallbackDashboardPreset,
             ];
         }
 
         return $this->dashboardPresets;
     }
 
-    /**
-     * @param DashboardPreset $dashboardPreset
-     */
     public function registerDashboardPreset(DashboardPreset $dashboardPreset): void
     {
         $this->dashboardPresets[$dashboardPreset->getIdentifier()] = $dashboardPreset;
