@@ -19,37 +19,51 @@ namespace TYPO3\CMS\Form\Domain\Condition\Functions;
 
 use Symfony\Component\ExpressionLanguage\ExpressionFunction;
 use Symfony\Component\ExpressionLanguage\ExpressionFunctionProviderInterface;
+use TYPO3\CMS\Extbase\Reflection\ObjectAccess;
 
 /**
  * @internal
  */
 class FormConditionFunctionsProvider implements ExpressionFunctionProviderInterface
 {
-
     /**
      * @return ExpressionFunction[] An array of Function instances
      */
-    public function getFunctions()
+    public function getFunctions(): array
     {
         return [
-            $this->getFormValueFunction()
+            $this->getFormValueFunction(),
+            $this->getRootFormPropertyFunction(),
         ];
     }
 
     /**
      * Shortcut function to access field values
-     *
-     * @return \Symfony\Component\ExpressionLanguage\ExpressionFunction
      */
     protected function getFormValueFunction(): ExpressionFunction
     {
         return new ExpressionFunction(
             'getFormValue',
-            function () {
-                // Not implemented, we only use the evaluator
-            },
-            function ($arguments, $field) {
-                return $arguments['formValues'][$field] ?? null;
+            static fn() => null, // Not implemented, we only use the evaluator
+            static function ($arguments, $field, $default = null) {
+                return $arguments['formValues'][$field] ?? $default;
+            }
+        );
+    }
+
+    protected function getRootFormPropertyFunction(): ExpressionFunction
+    {
+        return new ExpressionFunction(
+            'getRootFormProperty',
+            static fn() => null, // Not implemented, we only use the evaluator
+            static function ($arguments, $property) {
+                $formDefinition = $arguments['formRuntime']->getFormDefinition();
+                try {
+                    $value = ObjectAccess::getPropertyPath($formDefinition, $property);
+                } catch (\Exception) {
+                    $value = null;
+                }
+                return $value;
             }
         );
     }
