@@ -1,6 +1,6 @@
 <?php
-declare(strict_types = 1);
-namespace TYPO3\CMS\Form\Domain\Model\Exception;
+
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -15,12 +15,12 @@ namespace TYPO3\CMS\Form\Domain\Model\Exception;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Form\Domain\Model\Exception;
+
 use TYPO3\CMS\Form\Domain\Model\Exception;
 
 /**
  * This exception is thrown if a Validator Preset was not found,
  * or if the implementationClassName was not set.
  */
-class ValidatorPresetNotFoundException extends Exception
-{
-}
+class ValidatorPresetNotFoundException extends Exception {}
