@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 /*
  * This file is part of the TYPO3 CMS project.
  *
@@ -15,6 +17,7 @@
 
 namespace TYPO3\CMS\IndexedSearch\Utility;
 
+use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
@@ -24,42 +27,13 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
  */
 class IndexedSearchUtility
 {
-    /**
-     * Check if the tables provided are configured for usage. This becomes
-     * necessary for extensions that provide additional database functionality
-     * like indexed_search_mysql.
-     *
-     * @param string $tableName Table name to check
-     * @return bool True if the given table is used
-     */
-    public static function isTableUsed($tableName)
-    {
-        $tableList = $GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['indexed_search']['use_tables'];
-        return GeneralUtility::inList($tableList, $tableName);
-    }
-
-    /**
-     * md5 integer hash
-     * Using 7 instead of 8 just because that makes the integers lower than 32 bit (28 bit) and so they do not interfere with UNSIGNED integers or PHP-versions which has varying output from the hexdec function.
-     *
-     * @param string $stringToHash String to hash
-     * @return int Integer interpretation of the md5 hash of input string.
-     */
-    public static function md5inthash($stringToHash)
-    {
-        return hexdec(substr(md5($stringToHash), 0, 7));
-    }
-
     /**
      * Takes a search-string (WITHOUT SLASHES or else it'll be a little spooky , NOW REMEMBER to unslash!!)
      * Sets up search words with operators.
      *
      * @param string $sword The input search-word string.
-     * @param string $defaultOperator
-     * @param array $operatorTranslateTable
-     * @return array
      */
-    public static function getExplodedSearchString($sword, $defaultOperator, $operatorTranslateTable)
+    public static function getExplodedSearchString(string $sword, string $defaultOperator, array $operatorTranslateTable): array
     {
         $swordArray = [];
         $sword = trim($sword);
@@ -68,7 +42,7 @@ class IndexedSearchUtility
             if (is_array($components)) {
                 $i = 0;
                 $lastoper = '';
-                foreach ($components as $key => $val) {
+                foreach ($components as $val) {
                     $operator = self::getOperator($val, $operatorTranslateTable);
                     if ($operator) {
                         $lastoper = $operator;
@@ -92,9 +66,9 @@ class IndexedSearchUtility
      * @param string $origSword The raw sword string from outside
      * @param string $specchars Special chars which are used as operators (+- is default)
      * @param string $delchars Special chars which are deleted if the append the searchword (+-., is default)
-     * @return mixed Returns an ARRAY if there were search words, otherwise the return value may be unset.
+     * @return array<string>|null Returns an ARRAY if there were search words, otherwise the return value may be unset.
      */
-    protected static function split($origSword, $specchars = '+-', $delchars = '+.,-')
+    protected static function split(string $origSword, string $specchars = '+-', string $delchars = '+.,-'): ?array
     {
         $value = null;
         $sword = $origSword;
@@ -108,12 +82,12 @@ class IndexedSearchUtility
                 // Removes everything till next double-quote
                 preg_match('/^[^"]*/', $sword, $reg);
                 // reg[0] is the value, should not be trimmed
-                $value[] = $reg[0];
+                $value[] = (string)$reg[0];
                 $sword = (string)preg_replace('/^' . preg_quote($reg[0], '/') . '/', '', $sword);
                 // Removes last double-quote
                 $sword = trim((string)preg_replace('/^"/', '', $sword));
             } elseif (preg_match('/^' . $specs . '/', $sword, $reg)) {
-                $value[] = $reg[0];
+                $value[] = (string)$reg[0];
                 // Removes = sign
                 $sword = trim((string)preg_replace('/^' . $specs . '/', '', $sword));
             } elseif (preg_match('/[\\+\\-]/', $sword)) {
@@ -149,7 +123,7 @@ class IndexedSearchUtility
      * @param array $operatorTranslateTable an array of possible operators
      * @return string|null If found, the SQL operator for the localized input operator.
      */
-    protected static function getOperator($operator, $operatorTranslateTable)
+    protected static function getOperator(string $operator, array $operatorTranslateTable): ?string
     {
         $operator = trim($operator);
         // case-conversion is charset insensitive, but it doesn't spoil
@@ -168,12 +142,12 @@ class IndexedSearchUtility
     }
 
     /**
-     * Gets the unixtime as milliseconds.
-     *
-     * @return int The unixtime as milliseconds
+     * If MySQL Fulltext feature is used, the DB tables "index_rel" and "index_words" are not in use
+     * and skipped in various places.
      */
-    public static function milliseconds()
+    public static function isMysqlFullTextEnabled(): bool
     {
-        return round(microtime(true) * 1000);
+        $extConf = GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('indexed_search');
+        return (bool)($extConf['useMysqlFulltext'] ?? false);
     }
 }
