@@ -17,6 +17,7 @@ declare(strict_types=1);
 
 namespace TYPO3\CMS\Workspaces\Command;
 
+use Symfony\Component\Console\Attribute\AsCommand;
 use Symfony\Component\Console\Command\Command;
 use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Input\InputOption;
@@ -24,32 +25,39 @@ use Symfony\Component\Console\Output\OutputInterface;
 use Symfony\Component\Console\Style\SymfonyStyle;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\Core\Bootstrap;
+use TYPO3\CMS\Core\Database\Connection;
 use TYPO3\CMS\Core\Database\ConnectionPool;
 use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
 use TYPO3\CMS\Core\DataHandling\DataHandler;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
-use TYPO3\CMS\Core\Versioning\VersionState;
 
 /**
  * Fetches all versions in the database, and checks for integrity
+ *
+ * @internal
  */
+#[AsCommand('cleanup:versions', 'Find all versioned records and possibly cleans up invalid records in the database.')]
 class WorkspaceVersionRecordsCommand extends Command
 {
-
     /**
      * List of all workspaces
      * @var array
      */
     protected $allWorkspaces = [0 => 'Live Workspace'];
 
+    public function __construct(private readonly ConnectionPool $connectionPool)
+    {
+        parent::__construct();
+    }
+
     /**
      * Array with all records found when traversing the database
      * @var array
      */
     protected $foundRecords = [
         // All versions of records found
-        // Subset of "all" which are offline versions (pid=-1) [Informational]
+        // Subset of "all" which are offline versions (t3ver_oid > 0) [Informational]
         'all_versioned_records' => [],
         // All records that has been published and can therefore be removed permanently
         // Subset of "versions" that is a count of 1 or more (has been published) [Informational]
@@ -60,7 +68,7 @@ class WorkspaceVersionRecordsCommand extends Command
         'versions_in_live' => [],
         // Versions that has lost their connection to a workspace in TYPO3.
         // Subset of "versions" that doesn't belong to an existing workspace [Warning: Fix by move to live workspace]
-        'invalid_workspace' => []
+        'invalid_workspace' => [],
     ];
 
     /**
@@ -69,7 +77,6 @@ class WorkspaceVersionRecordsCommand extends Command
     public function configure()
     {
         $this
-            ->setDescription('Find all versioned records and possibly cleans up invalid records in the database.')
             ->setHelp('Traverse page tree and find versioned records. Also list all versioned records, additionally with some inconsistencies in the database, which can cleaned up with the "action" option. If you want to get more detailed information, use the --verbose option.')
             ->addOption(
                 'pid',
@@ -93,18 +100,14 @@ class WorkspaceVersionRecordsCommand extends Command
                 'action',
                 null,
                 InputOption::VALUE_OPTIONAL,
-                'Specify which action should be taken. Set it to "versions_in_live", "published_versions", "invalid_workspace" or "unused_placeholders"'
+                'Specify which action should be taken. Set it to "versions_in_live", "published_versions" or "invalid_workspace"'
             );
     }
 
     /**
      * Executes the command to find versioned records
-     *
-     * @param InputInterface $input
-     * @param OutputInterface $output
-     * @return int
      */
-    protected function execute(InputInterface $input, OutputInterface $output)
+    protected function execute(InputInterface $input, OutputInterface $output): int
     {
         // Make sure the _cli_ user is loaded
         Bootstrap::initializeBackendAuthentication();
@@ -146,16 +149,6 @@ class WorkspaceVersionRecordsCommand extends Command
             ksort($this->foundRecords[$kk]);
         }
 
-        $unusedPlaceholders = $this->findUnusedPlaceholderRecords();
-
-        // Finding all move placeholders with inconsistencies
-        // Move-to placeholder records which have bad integrity
-        $invalidMovePlaceholders = $this->findInvalidMovePlaceholderRecords();
-
-        // Finding move_id_check inconsistencies
-        // Checking if t3ver_move_id is correct. t3ver_move_id must only be set with online records having t3ver_state=3.
-        $recordsWithInvalidMoveIds = $this->findInvalidMoveIdRecords();
-
         if (!$io->isQuiet()) {
             $numberOfVersionedRecords = 0;
             foreach ($this->foundRecords['all_versioned_records'] as $records) {
@@ -205,21 +198,6 @@ class WorkspaceVersionRecordsCommand extends Command
                     $io->listing($records);
                 }
             }
-
-            $io->section('Found ' . count($unusedPlaceholders) . ' unused placeholder records.');
-            if ($io->isVeryVerbose()) {
-                $io->listing(array_keys($unusedPlaceholders));
-            }
-
-            $io->section('Found ' . count($invalidMovePlaceholders) . ' invalid move placeholders.');
-            if ($io->isVeryVerbose()) {
-                $io->listing($invalidMovePlaceholders);
-            }
-
-            $io->section('Found ' . count($recordsWithInvalidMoveIds) . ' versions with an invalid move ID.');
-            if ($io->isVeryVerbose()) {
-                $io->listing($recordsWithInvalidMoveIds);
-            }
         }
 
         // Actually permanently delete / update records
@@ -232,33 +210,26 @@ class WorkspaceVersionRecordsCommand extends Command
                 $this->deleteRecords($this->foundRecords['versions_in_live'], $dryRun, $io);
                 break;
 
-            // All records that has been published and can therefore be removed permanently
-            // Subset of "versions" that is a count of 1 or more (has been published)
+                // All records that has been published and can therefore be removed permanently
+                // Subset of "versions" that is a count of 1 or more (has been published)
             case 'published_versions':
                 $io->section('Deleting published records in live workspace now. ' . ($dryRun ? ' (Not deleting now, just a dry run)' : ''));
                 $this->deleteRecords($this->foundRecords['published_versions'], $dryRun, $io);
                 break;
 
-            // Versions that has lost their connection to a workspace in TYPO3.
-            // Subset of "versions" that doesn't belong to an existing workspace [Warning: Fix by move to live workspace]
+                // Versions that has lost their connection to a workspace in TYPO3.
+                // Subset of "versions" that doesn't belong to an existing workspace [Warning: Fix by move to live workspace]
             case 'invalid_workspace':
                 $io->section('Moving versions in invalid workspaces to live workspace now. ' . ($dryRun ? ' (Not deleting now, just a dry run)' : ''));
                 $this->resetRecordsWithoutValidWorkspace($this->foundRecords['invalid_workspace'], $dryRun, $io);
                 break;
 
-            // Finding all placeholders with no records attached
-            // Placeholder records which are not used anymore by offline versions.
-            case 'unused_placeholders':
-                $io->section('Deleting unused placeholder records now. ' . ($dryRun ? ' (Not deleting now, just a dry run)' : ''));
-                $this->deleteUnusedPlaceholders($unusedPlaceholders, $dryRun, $io);
-                break;
-
             default:
                 $io->note('No action specified, just displaying statistics. See --action option for details.');
                 break;
         }
         $io->success('All done!');
-        return 0;
+        return Command::SUCCESS;
     }
 
     /**
@@ -271,30 +242,25 @@ class WorkspaceVersionRecordsCommand extends Command
      */
     protected function traversePageTreeForVersionedRecords(int $rootID, int $depth, bool $isInsideVersionedPage = false, bool $rootIsVersion = false)
     {
-        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
+        $queryBuilder = $this->connectionPool->getQueryBuilderForTable('pages');
         $queryBuilder->getRestrictions()->removeAll();
 
         $pageRecord = $queryBuilder
             ->select(
                 'deleted',
                 'title',
-                't3ver_count',
                 't3ver_wsid'
             )
             ->from('pages')
-            ->where($queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter($rootID, \PDO::PARAM_INT)))
-            ->execute()
-            ->fetch();
+            ->where($queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter($rootID, Connection::PARAM_INT)))
+            ->executeQuery()
+            ->fetchAssociative();
 
         // If rootIsVersion is set it means that the input rootID is that of a version of a page. See below where the recursive call is made.
         if ($rootIsVersion) {
             $workspaceId = (int)$pageRecord['t3ver_wsid'];
             $this->foundRecords['all_versioned_records']['pages'][$rootID] = $rootID;
             // If it has been published and is in archive now...
-            if ($pageRecord['t3ver_count'] >= 1 && $workspaceId === 0) {
-                $this->foundRecords['published_versions']['pages'][$rootID] = $rootID;
-            }
-            // If it has been published and is in archive now...
             if ($workspaceId === 0) {
                 $this->foundRecords['versions_in_live']['pages'][$rootID] = $rootID;
             }
@@ -310,7 +276,7 @@ class WorkspaceVersionRecordsCommand extends Command
             foreach ($tableNames as $tableName) {
                 if ($tableName !== 'pages') {
                     // Select all records belonging to page:
-                    $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+                    $queryBuilder = $this->connectionPool
                         ->getQueryBuilderForTable($tableName);
 
                     $queryBuilder->getRestrictions()->removeAll();
@@ -321,24 +287,19 @@ class WorkspaceVersionRecordsCommand extends Command
                         ->where(
                             $queryBuilder->expr()->eq(
                                 'pid',
-                                $queryBuilder->createNamedParameter($rootID, \PDO::PARAM_INT)
+                                $queryBuilder->createNamedParameter($rootID, Connection::PARAM_INT)
                             )
                         )
-                        ->execute();
-                    while ($rowSub = $result->fetch()) {
+                        ->executeQuery();
+                    while ($rowSub = $result->fetchAssociative()) {
                         // Add any versions of those records
-                        $versions = BackendUtility::selectVersionsOfRecord($tableName, $rowSub['uid'], 'uid,t3ver_wsid,t3ver_count' . ($GLOBALS['TCA'][$tableName]['ctrl']['delete'] ? ',' . $GLOBALS['TCA'][$tableName]['ctrl']['delete'] : ''), null, true);
+                        $versions = BackendUtility::selectVersionsOfRecord($tableName, $rowSub['uid'], 'uid,t3ver_wsid' . (($GLOBALS['TCA'][$tableName]['ctrl']['delete'] ?? false) ? ',' . $GLOBALS['TCA'][$tableName]['ctrl']['delete'] : ''), null, true);
                         if (is_array($versions)) {
                             foreach ($versions as $verRec) {
-                                if (!$verRec['_CURRENT_VERSION']) {
+                                if (!($verRec['_CURRENT_VERSION'] ?? false)) {
                                     // Register version
                                     $this->foundRecords['all_versioned_records'][$tableName][$verRec['uid']] = $verRec['uid'];
                                     $workspaceId = (int)$verRec['t3ver_wsid'];
-                                    if ($verRec['t3ver_count'] >= 1 && $workspaceId === 0) {
-                                        // Only register published versions in LIVE workspace
-                                        // (published versions in draft workspaces are allowed)
-                                        $this->foundRecords['published_versions'][$tableName][$verRec['uid']] = $verRec['uid'];
-                                    }
                                     if ($workspaceId === 0) {
                                         $this->foundRecords['versions_in_live'][$tableName][$verRec['uid']] = $verRec['uid'];
                                     }
@@ -355,7 +316,7 @@ class WorkspaceVersionRecordsCommand extends Command
         // Find subpages to root ID and traverse (only when rootID is not a version or is a branch-version):
         if ($depth > 0) {
             $depth--;
-            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+            $queryBuilder = $this->connectionPool
                 ->getQueryBuilderForTable('pages');
 
             $queryBuilder->getRestrictions()->removeAll();
@@ -367,22 +328,22 @@ class WorkspaceVersionRecordsCommand extends Command
                 ->where(
                     $queryBuilder->expr()->eq(
                         'pid',
-                        $queryBuilder->createNamedParameter($rootID, \PDO::PARAM_INT)
+                        $queryBuilder->createNamedParameter($rootID, Connection::PARAM_INT)
                     )
                 )
                 ->orderBy('sorting');
 
-            $result = $queryBuilder->execute();
-            while ($row = $result->fetch()) {
+            $result = $queryBuilder->executeQuery();
+            while ($row = $result->fetchAssociative()) {
                 $this->traversePageTreeForVersionedRecords((int)$row['uid'], $depth, $isInsideVersionedPage, false);
             }
         }
         // Add any versions of pages
         if ($rootID > 0) {
-            $versions = BackendUtility::selectVersionsOfRecord('pages', $rootID, 'uid,t3ver_oid,t3ver_wsid,t3ver_count', null, true);
+            $versions = BackendUtility::selectVersionsOfRecord('pages', $rootID, 'uid,t3ver_oid,t3ver_wsid', null, true);
             if (is_array($versions)) {
                 foreach ($versions as $verRec) {
-                    if (!$verRec['_CURRENT_VERSION']) {
+                    if (!($verRec['_CURRENT_VERSION'] ?? false)) {
                         $this->traversePageTreeForVersionedRecords((int)$verRec['uid'], $depth, true, true);
                     }
                 }
@@ -390,160 +351,14 @@ class WorkspaceVersionRecordsCommand extends Command
         }
     }
 
-    /**
-     * Find all records where the field t3ver_state=1 (new placeholder)
-     *
-     * @return array the records (md5 as hash) with "table:uid" as value
-     */
-    protected function findUnusedPlaceholderRecords(): array
-    {
-        $unusedPlaceholders = [];
-        $tableNames = $this->getAllVersionableTables();
-        foreach ($tableNames as $table) {
-            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
-                ->getQueryBuilderForTable($table);
-
-            $queryBuilder->getRestrictions()
-                ->removeAll()
-                ->add(GeneralUtility::makeInstance(DeletedRestriction::class));
-
-            $result = $queryBuilder
-                ->select('uid', 'pid')
-                ->from($table)
-                ->where(
-                    $queryBuilder->expr()->gte('pid', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)),
-                    $queryBuilder->expr()->eq(
-                        't3ver_state',
-                        $queryBuilder->createNamedParameter(
-                            (string)new VersionState(VersionState::NEW_PLACEHOLDER),
-                            \PDO::PARAM_INT
-                        )
-                    )
-                )
-                ->execute();
-
-            while ($placeholderRecord = $result->fetch()) {
-                $versions = (array)BackendUtility::selectVersionsOfRecord($table, $placeholderRecord['uid'], 'uid', null);
-                if (count($versions) <= 1) {
-                    $unusedPlaceholders[$table . ':' . $placeholderRecord['uid']] = [
-                        'table' => $table,
-                        'uid'   => $placeholderRecord['uid']
-                    ];
-                }
-            }
-        }
-        ksort($unusedPlaceholders);
-        return $unusedPlaceholders;
-    }
-
-    /**
-     * Find all records where the field t3ver_state=3 (move placeholder)
-     * and checks against the ws_id etc.
-     *
-     * @return array the records (md5 as hash) with an array of data
-     */
-    protected function findInvalidMovePlaceholderRecords(): array
-    {
-        $invalidMovePlaceholders = [];
-        $tableNames = $this->getAllVersionableTables();
-        foreach ($tableNames as $table) {
-            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
-                ->getQueryBuilderForTable($table);
-
-            $queryBuilder->getRestrictions()
-                ->removeAll()
-                ->add(GeneralUtility::makeInstance(DeletedRestriction::class));
-
-            $result = $queryBuilder
-                ->select('uid', 'pid', 't3ver_move_id', 't3ver_wsid', 't3ver_state')
-                ->from($table)
-                ->where(
-                    $queryBuilder->expr()->gte('pid', $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)),
-                    $queryBuilder->expr()->eq(
-                        't3ver_state',
-                        $queryBuilder->createNamedParameter(
-                            (string)new VersionState(VersionState::MOVE_PLACEHOLDER),
-                            \PDO::PARAM_INT
-                        )
-                    )
-                )
-                ->execute();
-            while ($placeholderRecord = $result->fetch()) {
-                $shortID = GeneralUtility::shortMD5($table . ':' . $placeholderRecord['uid']);
-                if ((int)$placeholderRecord['t3ver_wsid'] !== 0) {
-                    $phrecCopy = $placeholderRecord;
-                    if (BackendUtility::movePlhOL($table, $placeholderRecord)) {
-                        if ($wsAlt = BackendUtility::getWorkspaceVersionOfRecord($phrecCopy['t3ver_wsid'], $table, $placeholderRecord['uid'], 'uid,pid,t3ver_state')) {
-                            if (!VersionState::cast($wsAlt['t3ver_state'])->equals(VersionState::MOVE_POINTER)) {
-                                $invalidMovePlaceholders[$shortID] = $table . ':' . $placeholderRecord['uid'] . ' - State for version was not "4" as it should be!';
-                            }
-                        } else {
-                            $invalidMovePlaceholders[$shortID] = $table . ':' . $placeholderRecord['uid'] . ' - No version was found for online record to be moved. A version must exist.';
-                        }
-                    } else {
-                        $invalidMovePlaceholders[$shortID] = $table . ':' . $placeholderRecord['uid'] . ' - Did not find online record for "t3ver_move_id" value ' . $placeholderRecord['t3ver_move_id'];
-                    }
-                } else {
-                    $invalidMovePlaceholders[$shortID] = $table . ':' . $placeholderRecord['uid'] . ' - Placeholder was not assigned a workspace value in t3ver_wsid.';
-                }
-            }
-        }
-        ksort($invalidMovePlaceholders);
-        return $invalidMovePlaceholders;
-    }
-
-    /**
-     * Find records with a t3ver_move_id field != 0 that are
-     * neither a move placeholder or, if it is a move placeholder is offline
-     *
-     * @return array
-     */
-    protected function findInvalidMoveIdRecords(): array
-    {
-        $records = [];
-        $tableNames = $this->getAllVersionableTables();
-        foreach ($tableNames as $table) {
-            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
-                ->getQueryBuilderForTable($table);
-
-            $queryBuilder->getRestrictions()
-                ->removeAll()
-                ->add(GeneralUtility::makeInstance(DeletedRestriction::class));
-
-            $result = $queryBuilder
-                ->select('uid', 'pid', 't3ver_move_id', 't3ver_wsid', 't3ver_oid', 't3ver_state')
-                ->from($table)
-                ->where(
-                    $queryBuilder->expr()->neq(
-                        't3ver_move_id',
-                        $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
-                    )
-                )
-                ->execute();
-
-            while ($placeholderRecord = $result->fetch()) {
-                if (VersionState::cast($placeholderRecord['t3ver_state'])->equals(VersionState::MOVE_PLACEHOLDER)) {
-                    if ((int)$placeholderRecord['t3ver_oid'] > 0) {
-                        $records[] = $table . ':' . $placeholderRecord['uid'] . ' - Record was offline, must not be!';
-                    }
-                } else {
-                    $records[] = $table . ':' . $placeholderRecord['uid'] . ' - Record had t3ver_move_id set to "' . $placeholderRecord['t3ver_move_id'] . '" while having t3ver_state=' . $placeholderRecord['t3ver_state'];
-                }
-            }
-        }
-        return $records;
-    }
-
     /**************************
      * actions / delete methods
      **************************/
-
     /**
      * Deletes records via DataHandler
      *
      * @param array $records two level array with tables and uids
      * @param bool $dryRun check if the records should NOT be deleted (use --dry-run to avoid)
-     * @param SymfonyStyle $io
      */
     protected function deleteRecords(array $records, bool $dryRun, SymfonyStyle $io)
     {
@@ -569,7 +384,7 @@ class WorkspaceVersionRecordsCommand extends Command
                     $io->writeln('Flushing record "' . $table . ':' . $uid . '"');
                 }
                 if (!$dryRun) {
-                    $dataHandler->deleteEl($table, $uid, true, true);
+                    $dataHandler->deleteAction($table, $uid, true, true);
                     if (!empty($dataHandler->errorLog)) {
                         $errorMessage = array_merge(['DataHandler reported an error'], $dataHandler->errorLog);
                         $io->error($errorMessage);
@@ -587,7 +402,6 @@ class WorkspaceVersionRecordsCommand extends Command
      *
      * @param array $records array with array of table and uid of each record
      * @param bool $dryRun check if the records should NOT be deleted (use --dry-run to avoid)
-     * @param SymfonyStyle $io
      */
     protected function resetRecordsWithoutValidWorkspace(array $records, bool $dryRun, SymfonyStyle $io)
     {
@@ -600,7 +414,7 @@ class WorkspaceVersionRecordsCommand extends Command
                     $io->writeln('Flushing record "' . $table . ':' . $uid . '"');
                 }
                 if (!$dryRun) {
-                    $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+                    $queryBuilder = $this->connectionPool
                         ->getQueryBuilderForTable($table);
 
                     $queryBuilder
@@ -608,11 +422,11 @@ class WorkspaceVersionRecordsCommand extends Command
                         ->where(
                             $queryBuilder->expr()->eq(
                                 'uid',
-                                $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)
+                                $queryBuilder->createNamedParameter($uid, Connection::PARAM_INT)
                             )
                         )
                         ->set('t3ver_wsid', 0)
-                        ->execute();
+                        ->executeStatement();
                     if (!$io->isQuiet()) {
                         $io->writeln('Flushed record "' . $table . ':' . $uid . '".');
                     }
@@ -621,36 +435,6 @@ class WorkspaceVersionRecordsCommand extends Command
         }
     }
 
-    /**
-     * Delete unused placeholders
-     *
-     * @param array $records array with array of table and uid of each record
-     * @param bool $dryRun check if the records should NOT be deleted (use --dry-run to avoid)
-     * @param SymfonyStyle $io
-     */
-    protected function deleteUnusedPlaceholders(array $records, bool $dryRun, SymfonyStyle $io)
-    {
-        $dataHandler = GeneralUtility::makeInstance(DataHandler::class);
-        $dataHandler->start([], []);
-        foreach ($records as $record) {
-            $table = $record['table'];
-            $uid = $record['uid'];
-            if ($io->isVeryVerbose()) {
-                $io->writeln('Deleting unused placeholder (soft) "' . $table . ':' . $uid . '"');
-            }
-            if (!$dryRun) {
-                $dataHandler->deleteAction($table, $uid);
-                // Return errors if any
-                if (!empty($dataHandler->errorLog)) {
-                    $errorMessage = array_merge(['DataHandler reported an error'], $dataHandler->errorLog);
-                    $io->error($errorMessage);
-                } elseif (!$io->isQuiet()) {
-                    $io->writeln('Permanently deleted unused placeholder "' . $table . ':' . $uid . '".');
-                }
-            }
-        }
-    }
-
     /**
      * HELPER FUNCTIONS
      */
@@ -662,7 +446,7 @@ class WorkspaceVersionRecordsCommand extends Command
      */
     protected function loadAllWorkspaceRecords(): array
     {
-        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
+        $queryBuilder = $this->connectionPool
             ->getQueryBuilderForTable('sys_workspace');
 
         $queryBuilder->getRestrictions()
@@ -672,9 +456,9 @@ class WorkspaceVersionRecordsCommand extends Command
         $result = $queryBuilder
             ->select('uid', 'title')
             ->from('sys_workspace')
-            ->execute();
+            ->executeQuery();
 
-        while ($workspaceRecord = $result->fetch()) {
+        while ($workspaceRecord = $result->fetchAssociative()) {
             $this->allWorkspaces[(int)$workspaceRecord['uid']] = $workspaceRecord['title'];
         }
         return $this->allWorkspaces;
@@ -682,8 +466,6 @@ class WorkspaceVersionRecordsCommand extends Command
 
     /**
      * Returns all TCA tables where workspaces is enabled
-     *
-     * @return array
      */
     protected function getAllVersionableTables(): array
     {
